# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0


from django.core.mail import EmailMultiAlternatives
from django.test import TestCase
from saferecipient import EmailBackend
import ssl


class TestEmailBackend(TestCase):
    SAFE_EMAIL = 'safe@example.com'

    def test_ssl_context(self):
        backend = EmailBackend()
        ssl_context = backend.ssl_context
        self.assertFalse(ssl_context.check_hostname)
        self.assertEqual(ssl_context.verify_mode, ssl.CERT_NONE)

    def test_safeguard(self):
        with self.settings(SAFE_EMAIL_RECIPIENT=self.SAFE_EMAIL):
            message = self._setup_message()
            EmailBackend()._safeguard(message)
            self.assertEqual(message.from_email, 'safe@example.com')
            self.assertEqual(message.to, ['safe@example.com'])
            self.assertEqual(message.cc, ['safe@example.com'])
            self.assertEqual(
                message.attachments[0].get_payload(),
                ("Original From: sender@example.com\nOriginal To: "
                 "['recipient@example.com']\nOriginal CC: "
                 "['cc_recipient@example.com']\nOriginal BCC: []\n"))

    def test_safeguard_with_safelist(self):
        with self.settings(SAFE_EMAIL_RECIPIENT=self.SAFE_EMAIL), \
             self.settings(SAFE_EMAIL_SAFELIST=[r".*@example\.com",
                                                r"safe.*@example\.org"]):
            message = self._setup_message(from_email='sender@example.com',
                                          to=['recipient@example.com'],
                                          cc=["safe+safe@example.org"],
                                          bcc=["unsafe@example.net"])
            EmailBackend()._safeguard(message)
            self.assertEqual(message.from_email, 'sender@example.com')
            self.assertEqual(message.to, ['recipient@example.com'])
            self.assertEqual(message.cc, ["safe+safe@example.org"])
            self.assertEqual(message.attachments[0].get_payload(),
                             ("Original From: sender@example.com\n"
                              "Original To: ['recipient@example.com']\n"
                              "Original CC: ['safe+safe@example.org']\n"
                              "Original BCC: ['unsafe@example.net']\n"))

    def test_no_setting(self):
        message = self._setup_message()
        self.assertRaises(
            AttributeError, EmailBackend()._safeguard, message)

    def test_only_safe_emails(self):
        with self.settings(SAFE_EMAIL_RECIPIENT=self.SAFE_EMAIL), \
             self.settings(SAFE_EMAIL_SAFELIST=[r".*@example\.com",
                                                r"safe.*@example\.org"]):
            eb = EmailBackend()
            self.assertEqual(([], False), eb._only_safe_emails([]))
            self.assertEqual(([self.SAFE_EMAIL], True),
                             eb._only_safe_emails(["unsafe@example.org"]))
            self.assertEqual(([self.SAFE_EMAIL], True),
                             eb._only_safe_emails(["unsafe@example.org",
                                                   "unsafe2@example.org"]))
            self.assertEqual(([self.SAFE_EMAIL], True),
                             eb._only_safe_emails([self.SAFE_EMAIL,
                                                   "unsafe@example.org",
                                                   "unsafe2@example.org"]))
            self.assertEqual((["safe+2@example.com", self.SAFE_EMAIL], True),
                             eb._only_safe_emails(["safe+2@example.com",
                                                   self.SAFE_EMAIL,
                                                   "unsafe@example.org",
                                                   "unsafe2@example.org"]))
            self.assertEqual((["safe+2@example.com",
                               self.SAFE_EMAIL,
                               "safe@example.org"],
                              True),
                             eb._only_safe_emails(["safe+2@example.com",
                                                   self.SAFE_EMAIL,
                                                   "unsafe@example.org",
                                                   "unsafe2@example.org",
                                                   "safe@example.org"]))

    def test_safelist(self):
        with self.settings(SAFE_EMAIL_SAFELIST=[r".*@example\.com",
                                                r"safe.*@example\.org"]):
            eb = EmailBackend()
            self.assertTrue(eb._is_safelisted("example@example.com"))
            self.assertTrue(eb._is_safelisted("example2@example.com"))
            self.assertTrue(eb._is_safelisted("safe@example.org"))
            self.assertTrue(eb._is_safelisted("safe+2@example.org"))
            self.assertFalse(eb._is_safelisted("unsafe@example.org"))

    def _setup_message(self, from_email='sender@example.com', to=None,
                       cc=None, bcc=None):
        if to is None:
            to = ['recipient@example.com']
        if cc is None:
            cc = ['cc_recipient@example.com']
        if bcc is None:
            bcc = []
        return EmailMultiAlternatives(from_email=from_email, to=to, cc=cc,
                                      bcc=bcc, subject='test', body='TEST')



# Copyright 2024 UW-IT, University of Washington
# SPDX-License-Identifier: Apache-2.0

import re
import ssl
from django.conf import settings
from django.core.mail.backends.smtp import EmailBackend as SMTPEmailBackend
from django.utils.functional import cached_property
from email.mime.text import MIMEText


class EmailBackend(SMTPEmailBackend):
    """
    An SMTP email backend class that only sends email to a safe email address.

    Re-routes email, so it goes to, and comes from
    settings.SAFE_EMAIL_RECIPIENT.

    The original to and from are added to a text file
    that is attached to the message.
    """

    @cached_property
    def ssl_context(self):
        """
        See https://code.djangoproject.com/ticket/34504
        """
        try:
            ssl_context = super().ssl_context
        except AttributeError:  # Django < 4
            ssl_context = ssl.SSLContext()
        ssl_context.check_hostname = False
        ssl_context.verify_mode = ssl.CERT_NONE
        return ssl_context

    def send_messages(self, email_messages):
        for message in email_messages:
            self._safeguard(message)
        return super().send_messages(email_messages)

    def _safeguard(self, message):
        originals = ("Original From: {}\nOriginal To: {}\nOriginal CC: {}\n"
                     "Original BCC: {}\n").format(
                         message.from_email, message.to,
                         message.cc, message.bcc)

        from_modified = False
        if not self._is_safelisted(message.from_email):
            from_modified = True
            message.from_email = settings.SAFE_EMAIL_RECIPIENT

        message.to, to_modified = self._only_safe_emails(message.to)
        message.cc, cc_modified = self._only_safe_emails(message.cc)
        message.bcc, bcc_modified = self._only_safe_emails(message.bcc)

        if from_modified or to_modified or cc_modified or bcc_modified:
            text_attachment = MIMEText(originals)
            text_attachment.add_header(
                'Content-disposition',
                'attachment; filename="original_emails.txt"')
            message.attach(text_attachment)

    def _only_safe_emails(self, emails):
        """"
        Given a list of emails, checks whether they are all in the safe list.
        """
        email_modified = False
        if any(not self._is_safelisted(email) for email in emails):
            email_modified = True
            emails = [email for email in emails if self._is_safelisted(email)]
            if settings.SAFE_EMAIL_RECIPIENT not in emails:
                emails.append(settings.SAFE_EMAIL_RECIPIENT)
        return emails, email_modified

    def _is_safelisted(self, email):
        """
        Check if an email is in the safelist. If there's no safelist,
        it's assumed it's not safelisted.
        """
        return (hasattr(settings, "SAFE_EMAIL_SAFELIST") and
                any(re.match(m, email) for m in settings.SAFE_EMAIL_SAFELIST))
